      DOUBLE PRECISION FUNCTION DMLIN(F,A,B,MAXFCN,RERR,IER)



C	LATEST REVISION	- JULY, 1999

C	PURPOSE			- TO INTEGRATE A FUNCTION VIA THE METHOD OF GAUSSIAN
C					  QUADRATURE.  IF INTEGRAL CONVERGENCE IS NOT
C					  ACHIEVED, THIS FUNCTION TERMINTATES WHEN EITHER
C					  THE MAXIMUM NUMBER OF FUNCTION EVALUATIONS IS
C					  PERFORMED OR MORE THAN MAXFCN GAUSSIAN POINTS ARE
C					  EVALUATED. THIS FUNCTION IS BASED UPON THE
C					  ROUTINE XGAULEG FOUND IN "NUMERICAL RECIPIES
C					  EXAMPLE BOOK(FORTRAN)", SECOND EDITION, CAMBRIDGE
C					  UNIVERSITY PRESS, P. 48.

C	SUBROUTINES 	- 
C		GAULEG		  THIS SUBROUTINE IS BASED UPON THE SUBROUTINE IN
C				      "NUMERICAL RECIPIES IN FORTRAN 77", SECOND EDITION,
C					  CAMBRIDGE UNIVERSITY PRESS, P.145.  IT RETURNS
C		              ARRAYS x(1:n) AND w(1:n) OF LENGTH n, CONTAINING
C				      THE ABSCISSAS AND WEIGHTS OF THE GAUSSIAN-LEGENDRE
C					  n-POINT QUADRATURE FORMULA.  x1 and x2 ARE THE
C					  LOWER AND UPPER LIMITS OF INTEGRATION, RESP.	

C	FUNCTIONS		-
C		F(x)		  USER SUPPLIED FUNCTION SUBPROGRAM WHICH
C					  DEFINES FUNCTION TO BE INTEGRATED, DECLARED 
C					  EXTERNAL IN THE CALLING PROGRAM 

C	ERROR CODES		- 120 MEANS THAT CONVERGENCE CRITERIA HAS NOT BEEN 
C					  MET AFTER MAXIMUM NUMBER OF ITERATIONS HAVE BEEN 
C					  PERFORMED AND FUNCTION TERMINATES
                        
	


	IMPLICIT NONE
					
	INTEGER*4	MAXFCN, IER, i, IRES, NLIM, NPOINT

	REAL*8		A, B, F, RERR, LASTSUM, INTSUM, x(MAXFCN), w(MAXFCN)
                          
		DMLIN = 0.0d0
	
C		Maximum possible order of the Gaussian quadrature integration 
C		is 2**NLIM

		NLIM = 12

		DO  IRES = 1,NLIM
			LASTSUM = INTSUM
			NPOINT = 2**IRES
			IF (NPOINT .GT. MAXFCN) GOTO 50
C			use GAULEG to perform the integration
			call gauleg(A,B,x,w,NPOINT)
			INTSUM = 0.0d0
			DO i = 1,NPOINT
				INTSUM = INTSUM+w(i)*F(x(i))
			END DO 
C			use last two values to check for convergence
			IF ((IRES .GT. 1) .AND. (DABS(INTSUM-LASTSUM).LE.                      
     *		RERR*DABS(INTSUM))) GOTO 60
		END DO 
		 
C		IER = 120 indicates no convergence                                                       
   50		IER = 120 

	RETURN

C		IER = 0 indicates convergence
   60		IER = 0                                                       
		DMLIN = INTSUM 
		 
      END                                                          
           
		 
      SUBROUTINE	GAULEG(x1,x2,x,w,n)

	IMPLICIT NONE

      INTEGER*4	n, i, j, m

      REAL*8		EPS, x1, x2, x(n), w(n)

      PARAMETER	(EPS=3.d-14)

      REAL*8		p1, p2, p3, pp, xl, xm, z, z1

		m=(n+1)/2
		xm=0.5d0*(x2+x1)
		xl=0.5d0*(x2-x1)

		DO 12 i=1,m
			z=cos(3.141592654d0*(i-.25d0)/(n+.5d0))
    1			CONTINUE
			p1=1.d0
			p2=0.d0
			DO 11 j=1,n
				p3=p2
				p2=p1
				p1=((2.d0*j-1.d0)*z*p2-(j-1.d0)*p3)/j
   11			CONTINUE
		pp=n*(z*p1-p2)/(z*z-1.d0)
		z1=z

          z=z1-p1/pp
		IF(DABS(z-z1).gt.EPS) GOTO 1
		x(i)=xm-xl*z
		x(n+1-i)=xm+xl*z
		w(i)=2.d0*xl/((1.d0-z*z)*pp*pp)
		w(n+1-i)=w(i)
   12		CONTINUE

	RETURN
	END

		                                                          
